@@ -9,8 +9,9 @@ defaults if PyICU is not found.
 """
 
 __license__ = """
-Copyright (c) 2004-2007 Mike Taylor
-Copyright (c) 2006-2007 Darshana Chhajed
+Copyright (c) 2004-2008 Mike Taylor
+Copyright (c) 2006-2008 Darshana Chhajed
+Copyright (c)      2007 Bernd Zeimetz <bzed@debian.org>
 All rights reserved.
 
 Licensed under the Apache License, Version 2.0 (the "License");
@@ -33,6 +34,7 @@ except:
 
 
 import datetime
+import calendar
 import time
 import re
 
@@ -112,7 +114,6 @@ class pdtLocale_en:
                   'prev':      -1,
                   'last':      -1,
                   'next':       1,
-                  'this':       0,
                   'previous':  -1,
                   'in a':       2,
                   'end of':     0,
@@ -217,7 +218,6 @@ class pdtLocale_au:
                   'prev':      -1,
                   'last':      -1,
                   'next':       1,
-                  'this':       0,
                   'previous':  -1,
                   'in a':       2,
                   'end of':     0,
@@ -323,7 +323,6 @@ class pdtLocale_es:
                   'prev':     -1,
                   'last':     -1,
                   'next':      1,
-                  'this':      0,
                   'previous': -1,
                   'in a':      2,
                   'end of':    0,
@@ -352,9 +351,128 @@ class pdtLocale_es:
                     }
 
 
+class pdtLocale_de:
+    """
+    de_DE Locale constants
+
+    This class will be used to initialize L{Constants} if PyICU is not located.
+
+    Contributed by Debian parsedatetime package maintainer Bernd Zeimetz <bzed@debian.org>
+
+    Defined as class variables are the lists and strings needed by parsedatetime
+    to evaluate strings for German
+    """
+
+    localeID      = 'de_DE'   # don't use a unicode string
+    dateSep       = [ u'.' ]
+    timeSep       = [ u':' ]
+    meridian      = [ ]
+    usesMeridian  = False
+    uses24        = True
+
+    Weekdays      = [ u'montag', u'dienstag', u'mittwoch',
+                      u'donnerstag', u'freitag', u'samstag', u'sonntag',
+                    ]
+    shortWeekdays = [ u'mo', u'di', u'mi',
+                      u'do', u'fr', u'sa', u'so',
+                    ]
+    Months        = [ u'januar',  u'februar',  u'm\xe4rz',
+                      u'april',   u'mai',      u'juni',
+                      u'juli',    u'august',   u'september',
+                      u'oktober', u'november', u'dezember',
+                    ]
+    shortMonths   = [ u'jan', u'feb', u'mrz',
+                      u'apr', u'mai', u'jun',
+                      u'jul', u'aug', u'sep',
+                      u'okt', u'nov', u'dez',
+                    ]
+    dateFormats   = { 'full':   u'EEEE, d. MMMM yyyy',
+                      'long':   u'd. MMMM yyyy',
+                      'medium': u'dd.MM.yyyy',
+                      'short':  u'dd.MM.yy'
+                    }
+
+    timeFormats   = { 'full':   u'HH:mm:ss v',
+                      'long':   u'HH:mm:ss z',
+                      'medium': u'HH:mm:ss',
+                      'short':  u'HH:mm'
+                    }
+
+    dp_order = [ u'd', u'm', u'y' ]
+
+      # this will be added to re_consts later
+    units = { 'seconds': [ 'sekunden', 'sek',  's' ],
+              'minutes': [ 'minuten',  'min' , 'm' ],
+              'hours':   [ 'stunden',  'std',  'h' ],
+              'days':    [ 'tage',     't' ],
+              'weeks':   [ 'wochen',   'w' ],
+              'months':  [ 'monate' ], #the short version would be a capital M,
+                                       #as I understand it we can't distinguis
+                                       #between m for minutes and M for months.
+              'years':   [ 'jahre',    'j' ],
+            }
+
+      # text constants to be used by regex's later
+    re_consts     = { 'specials':       'am|dem|der|im|in|den|zum',
+                      'timeseperator':  ':',
+                      'rangeseperator': '-',
+                      'daysuffix':      '',
+                      'qunits':         'h|m|s|t|w|m|j',
+                      'now':            [ 'jetzt' ],
+                    }
+
+      # Used to adjust the returned date before/after the source
+      #still looking for insight on how to translate all of them to german.
+    modifiers = { u'from':         1,
+                  u'before':      -1,
+                  u'after':        1,
+                  u'vergangener': -1,
+                  u'vorheriger':  -1,
+                  u'prev':        -1,
+                  u'letzter':     -1,
+                  u'n\xe4chster':  1,
+                  u'dieser':       0,
+                  u'previous':    -1,
+                  u'in a':         2,
+                  u'end of':       0,
+                  u'eod':          0,
+                  u'eo':           0,
+                }
+
+     #morgen/abermorgen does not work, see http://code.google.com/p/parsedatetime/issues/detail?id=19
+    dayoffsets = { u'morgen':        1,
+                   u'heute':         0,
+                   u'gestern':      -1,
+                   u'vorgestern':   -2,
+                   u'\xfcbermorgen': 2,
+                 }
+
+      # special day and/or times, i.e. lunch, noon, evening
+      # each element in the dictionary is a dictionary that is used
+      # to fill in any value to be replace - the current date/time will
+      # already have been populated by the method buildSources
+    re_sources    = { u'mittag':      { 'hr': 12, 'mn': 0, 'sec': 0 },
+                      u'mittags':     { 'hr': 12, 'mn': 0, 'sec': 0 },
+                      u'mittagessen': { 'hr': 12, 'mn': 0, 'sec': 0 },
+                      u'morgen':      { 'hr':  6, 'mn': 0, 'sec': 0 },
+                      u'morgens':     { 'hr':  6, 'mn': 0, 'sec': 0 },
+                      u'fr\e4hst\xe4ck': { 'hr':  8, 'mn': 0, 'sec': 0 },
+                      u'abendessen':  { 'hr': 19, 'mn': 0, 'sec': 0 },
+                      u'abend':       { 'hr': 18, 'mn': 0, 'sec': 0 },
+                      u'abends':      { 'hr': 18, 'mn': 0, 'sec': 0 },
+                      u'mitternacht': { 'hr':  0, 'mn': 0, 'sec': 0 },
+                      u'nacht':       { 'hr': 21, 'mn': 0, 'sec': 0 },
+                      u'nachts':      { 'hr': 21, 'mn': 0, 'sec': 0 },
+                      u'heute abend': { 'hr': 21, 'mn': 0, 'sec': 0 },
+                      u'heute nacht': { 'hr': 21, 'mn': 0, 'sec': 0 },
+                      u'feierabend':  { 'hr': 17, 'mn': 0, 'sec': 0 },
+                    }
+
+
 pdtLocales = { 'en_US': pdtLocale_en,
                'en_AU': pdtLocale_au,
-               'es':    pdtLocale_es,
+               'es_ES': pdtLocale_es,
+               'de_DE': pdtLocale_de,
              }
 
 
@@ -628,7 +746,7 @@ def _initPatterns(ptc):
                                    ))''' % ptc.re_values
     ptc.RE_MODIFIER  = r'''(\s?|^)
                            (?P<modifier>
-                            (previous|prev|last|next|this|eod|eo|(end\sof)|(in\sa)))''' % ptc.re_values
+                            (previous|prev|last|next|eod|eo|(end\sof)|(in\sa)))''' % ptc.re_values
     ptc.RE_MODIFIER2 = r'''(\s?|^)
                            (?P<modifier>
                             (from|before|after|ago|prior))
@@ -706,6 +824,10 @@ def _initPatterns(ptc):
     ptc.TIMERNG3 = r'\d\d?\s?%(rangeseperator)s\s?' + ptc.RE_RTIMEHMS2
     ptc.TIMERNG3 = ptc.TIMERNG3 % ptc.re_values
 
+    # "4:30-5pm "
+    ptc.TIMERNG4 = ptc.RE_RTIMEHMS + r'\s?%(rangeseperator)s\s?' + ptc.RE_RTIMEHMS2
+    ptc.TIMERNG4 = ptc.TIMERNG4 % ptc.re_values
+
 
 def _initConstants(ptc):
     """
@@ -727,17 +849,14 @@ def _initConstants(ptc):
       # build month offsets - yes, it assumes the Months and shortMonths
       # lists are in the same order and Jan..Dec
     ptc.MonthOffsets = {}
-    ptc.DaysInMonth  = {}
 
     o = 1
     for key in ptc.Months:
         ptc.MonthOffsets[key] = o
-        ptc.DaysInMonth[key]  = ptc.DaysInMonthList[o - 1]
         o += 1
     o = 1
     for key in ptc.shortMonths:
         ptc.MonthOffsets[key] = o
-        ptc.DaysInMonth[key]  = ptc.DaysInMonthList[o - 1]
         o += 1
 
 
@@ -768,6 +887,15 @@ class Constants:
         self.locale   = None
         self.usePyICU = usePyICU
 
+        # starting cache of leap years
+        # daysInMonth will add to this if during
+        # runtime it gets a request for a year not found
+        self._leapYears = [ 1904, 1908, 1912, 1916, 1920, 1924, 1928, 1932, 1936, 1940, 1944,
+                            1948, 1952, 1956, 1960, 1964, 1968, 1972, 1976, 1980, 1984, 1988,
+                            1992, 1996, 2000, 2004, 2008, 2012, 2016, 2020, 2024, 2028, 2032,
+                            2036, 2040, 2044, 2048, 2052, 2056, 2060, 2064, 2068, 2072, 2076,
+                            2080, 2084, 2088, 2092, 2096 ]
+
         self.Second =   1
         self.Minute =  60 * self.Second
         self.Hour   =  60 * self.Minute
@@ -778,7 +906,7 @@ class Constants:
 
         self.rangeSep = u'-'
 
-        self.DaysInMonthList = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)
+        self._DaysInMonthList = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)
 
         self.BirthdayEpoch = 50
 
@@ -841,7 +969,6 @@ class Constants:
         self.dayOffsets     = None
         self.WeekdayOffsets = None
         self.MonthOffsets   = None
-        self.DaysInMonth    = None
         self.dateSep        = None
         self.timeSep        = None
         self.am             = None
@@ -876,6 +1003,7 @@ class Constants:
         self.TIMERNG1     = r''
         self.TIMERNG2     = r''
         self.TIMERNG3     = r''
+        self.TIMERNG4     = r''
 
         _initLocale(self)
         _initConstants(self)
@@ -905,6 +1033,7 @@ class Constants:
                             'CRE_TIMERNG1':  self.TIMERNG1,
                             'CRE_TIMERNG2':  self.TIMERNG2,
                             'CRE_TIMERNG3':  self.TIMERNG3,
+                            'CRE_TIMERNG4':  self.TIMERNG4,
                             'CRE_DATERNG1':  self.DATERNG1,
                             'CRE_DATERNG2':  self.DATERNG2,
                             'CRE_DATERNG3':  self.DATERNG3,
@@ -920,6 +1049,25 @@ class Constants:
         else:
             raise AttributeError, name
 
+    def daysInMonth(self, month, year):
+        """
+        Take the given month (1-12) and a given year (4 digit) return
+        the number of days in the month adjusting for leap year as needed
+        """
+        result = None
+
+        if month > 0 and month <= 12:
+            result = self._DaysInMonthList[month - 1]
+
+            if month == 2:
+                if year in self._leapYears:
+                    result += 1
+                else:
+                    if calendar.isleap(year):
+                        self._leapYears.append(year)
+                        result += 1
+
+        return result
 
     def buildSources(self, sourceTime=None):
         """
