@@ -20,6 +20,15 @@ def _compareResults(result, check):
     return ((t_yr == v_yr) and (t_mth == v_mth) and (t_dy == v_dy) and
             (t_hr == v_hr) and (t_min == v_min)) and (t_flag == v_flag)
 
+def _compareResultsErrorFlag(result, check):
+    target, t_flag = result
+    value,  v_flag = check
+
+    t_yr, t_mth, t_dy, _, _, _, _, _, _ = target
+    v_yr, v_mth, v_dy, _, _, _, _, _, _ = value
+
+    return ((t_yr <> v_yr) and (t_mth <> v_mth) and (t_dy <> v_dy)) and (t_flag == v_flag)
+
 class test(unittest.TestCase):
     def setUp(self):
         self.cal = pt.Calendar()
@@ -29,6 +38,7 @@ class test(unittest.TestCase):
         s     = datetime.datetime.now()
         start = s.timetuple()
 
+        # These tests all return current date/time as they are out of range
         self.assertTrue(_compareResults(self.cal.parse('01/0',   start), (start, 0)))
         self.assertTrue(_compareResults(self.cal.parse('08/35',  start), (start, 0)))
         self.assertTrue(_compareResults(self.cal.parse('18/35',  start), (start, 0)))
@@ -41,7 +51,8 @@ class test(unittest.TestCase):
         self.assertTrue(_compareResults(self.cal.parse('174565', start), (start, 0)))
         self.assertTrue(_compareResults(self.cal.parse('177505', start), (start, 0)))
 
-        self.assertTrue(_compareResults(self.cal.parse('30/030/01/071/07', start), (start, 0)))
+        # This test actually parses into *something* for some locales, so need to check the error flag
+        self.assertTrue(_compareResultsErrorFlag(self.cal.parse('30/030/01/071/07', start), (start, 1)))
 
 
 if __name__ == "__main__":
